<?php

require(DOCROOT.'/modules/mail/vendor/PHPMailer/PHPMailerAutoload.php');

function gradeSendMail($subject, $body, $sendToEmail, $sendToName) {
    $mail = new PHPMailer;

    //$mail->SMTPDebug = 3;                               // Enable verbose debug output

    $mail->isSMTP();                                      // Set mailer to use SMTP
    $mail->Host = 'class.mmcs.sfedu.ru';  // Specify main and backup SMTP servers
    $mail->SMTPAuth = false;                               // Enable SMTP authentication
    $mail->Username = NULL;                 // SMTP username
    $mail->Password = NULL;                           // SMTP password
    $mail->SMTPSecure = NULL;                            // Enable TLS encryption, `ssl` also accepted
    $mail->Port = 25;                                    // TCP port to connect to

    $mail->From = 'no_reply@grade.sfedu.ru';
    $mail->FromName = 'Grade System (Сервис БРС)';
    $mail->addAddress($sendToEmail, $sendToName);     // Add a recipient
    //$mail->addAddress('ellen@example.com');               // Name is optional
    //$mail->addReplyTo('info@example.com', 'Information');
    //$mail->addCC('cc@example.com');
    //$mail->addBCC('bcc@example.com');

    //$mail->addAttachment('/var/tmp/file.tar.gz');         // Add attachments
    //$mail->addAttachment('/tmp/image.jpg', 'new.jpg');    // Optional name
    //$mail->isHTML(true);                                  // Set email format to HTML

    $mail->Subject = $subject;
    $mail->Body    = $body;
    //$mail->ContentType = 'text/plain';
    $mail->CharSet = 'utf-8';
    //$mail->AltBody = 'This is the body in plain text for non-HTML mail clients';

    if(!$mail->send()) {
        Log::instance()->add(Log::NOTICE, 'Message could not be sent: '.$mail->ErrorInfo);
        //echo 'Mailer Error: ' . $mail->ErrorInfo;
    } else {
        Log::instance()->add(Log::NOTICE, 'Message has been sent');
    }
}

class Kohana_Account {
    
    protected static $_instance;
    protected $_config;
    protected $_model;

    /**
     * Вовзращает экземпляр класса (singleton-паттерн)
     * @return self
     */
    public static function instance() {
        if (!isset(self::$_instance)) {
            $config = Kohana::$config->load('account');
            self::$_instance = new self($config);
        }
        return self::$_instance;
    }
    
    private function __construct($config = array()) {
        $this->_config = $config;
        $this->_model = new Model_Account;
    }

    private function generateActivationCode() {
        $activationCode = Text::random('ABDCEFGHJKLMNPQRSTUVWXYZ123456789', 10);
        return UTF8::strtoupper($activationCode);
    }

    public function createTeacher($lastName, $firstName, $secondName, $degreeID, $departmentID) {
        $code = $this->generateActivationCode();
        $response = $this->_model->createTeacher($lastName, $firstName, $secondName, $degreeID, $departmentID, $code);
        return $response == -1 ? -1 : $code;
    }

    public function createTeacherByDepName($lastName, $firstName, $secondName, $departmentName, $facultyID) {
        $code = $this->generateActivationCode();
        $response = $this->_model->createTeacherByDepName($lastName, $firstName, $secondName, $departmentName, $facultyID, $code);
        return $response == -1 ? -1 : $code;
    }

    public function createStudent($lastName, $firstName, $secondName, $grade, $groupNum, $facultyID) {
        $code = $this->generateActivationCode();
        $response = $this->_model->createStudent($lastName, $firstName, $secondName, $grade, $groupNum, $facultyID, $code);
        return $response == -1 ? -1 : $code;
    }

    public function createStudentEx($lastName, $firstName, $secondName, $grade, $groupNum, $degree, $spec, $facultyID) {
        $code = $this->generateActivationCode();
        $response = $this->_model->createStudentEx($lastName, $firstName, $secondName, $grade, $groupNum, $degree, $spec, $facultyID, $code);
        return $response == -1 ? -1 : $code;
    }

    public function createSubject($name, $abbr, $facultyID) {
        return $this->_model->createSubject($name, $abbr, $facultyID);
    }

    private function checkTokenLifetime($creationDate) {
        $config = Kohana::$config->load('security.securityPolicy');
        $lifetime = $config['recoveryToken']['lifetime'];
        return (time() - $creationDate) > $lifetime;
    }

    public function checkToken($token) {
        $recovery = $this->_model->getRecoveryInfoByToken($token)[0];
        $response = true;

        if ($recovery['isUsed']) {
            $response = false;
        } else {
            $date = strtotime($recovery['Date']);
            if ($this->checkTokenLifetime($date)) {
                $this->_model->useRecoveryToken($recovery['Token']);
                $response = false;
            }
        }
        return $response;
    }

    public function createRecoveryRequest($email) {
        $requestToken = sha1($email.time().Cookie::$salt);

        $UserFullName = $this->_model->createRecoveryToken($email, $requestToken);
        if (!$UserFullName) {
            throw HTTP_Exception::factory(403, 'Пользователь с таким e-mail адресом не зарегистрирован в системе!');
        }

        $subject = ASSEMBLY_SYSTEM_NAME . ": Восстановление пароля";

        $twig = Twig::factory('email/recovery');
        $twig->curl = 'https://grade.sfedu.ru/'; // URL::base('https', TRUE); TODO: fix after mmcs.sfedu.ru gets SSL-certificate
        $twig->Token = $requestToken;
        $twig->EMail = $email;
        $twig->Subject = $subject;

        /* Kohana mailer is not working
        $status = Mailer::factory()
            ->headers('Content-Type', 'text/html; charset=utf-8')
            ->subject($subject)
            ->in_reply_to(Mailer::message_id())
            ->from('no-reply@rating.mmcs.sfedu.ru')
            ->body($twig->render())
            ->send($email);*/
        gradeSendMail($subject, $twig->render(), $email, $UserFullName);
    }

    public function changePasswordByToken($token, $password) {
        $recovery = $this->_model->getRecoveryInfoByToken($token)[0];
        $this->changePassword($recovery['AccountID'], $password);
        $this->_model->useRecoveryToken($token);
    }

    public function loginExists($login) {
        $login_count = $this->_model->getAccNumByLogin($login);
        return $login_count > 0;
    }

    public function mailExists($email) {
        $email_count = $this->_model->getAccNumByMail($email);
        return $email_count > 0;
    }

    public function changeLogin($id, $newLogin) {
        $response = $this->_model->changeLogin($id, $newLogin);
        return $response != -1;
    }

    public function changeEMail($id, $newEMail) {
        $response = $this->_model->changeMail($id, $newEMail);
        return $response != -1;
    }

    public function changePassword($id, $newPassword) {
        $response = $this->_model->changePassword($id, $newPassword);
        return $response != -1;
    }
}